# -*- coding: utf-8 -*-
# Description:
# Manage interaction with Acunetix process
#
# Authors:
# Benoît Allard <benoit.allard@greenbone.net>
#
# Copyright:
# Copyright (C) 2015 Greenbone Networks GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import logging
import os
import shutil
import subprocess
import tempfile

from ospd_acunetix.parser import Parser

logger = logging.getLogger(__name__)


class Acunetix(object):
    """ A class to manage interaction with the Acunetix process """
    # Return code when the scanner stops unexpectedly
    UNEXPECTED_ERROR = 666
    EXE_PATH = "c:\Program Files\Acunetix\Web Vulnerability Scanner 10"

    def __init__(self, exe_path=None):
        if exe_path is not None:
            self.exe_path = exe_path
        else:
            self.exe_path = self.EXE_PATH
        self.tmpdir = tempfile.mkdtemp(prefix="acunetix-scan-")

    def run_scan(self, target, profile=None):
        command = [os.path.join(self.exe_path, 'wvs_console.exe')]
        command.extend(["/Scan", target])
        if profile:  # Accept None and '' as *don't set it*
            command.extend(["/Profile", profile])

        command.extend(["/SaveFolder", self.tmpdir])
        command.extend(["/Save", "/SaveLogs", "/ExportXML"])

        stdout = open(os.path.join(self.tmpdir, 'stdout.txt'), 'wt')
        stderr = open(os.path.join(self.tmpdir, 'stderr.txt'), 'wt')

        logger.debug("Command used: %s", ' '.join(command))
        ret = subprocess.call(command, stdout=stdout, stderr=stderr)
        stdout.close()
        stderr.close()
        logger.debug("Acunetix returned: %d", ret)
        if ret == self.UNEXPECTED_ERROR:
            raise subprocess.CalledProcessError("Scanner stopped unexpectedly")

        return Parser(os.path.join(self.tmpdir, 'export.xml'))

    def cleanup(self):
        shutil.rmtree(self.tmpdir)
